import { Order } from '@project-serum/serum/lib/market';
declare type SpotMarketSymbol = string;
declare type OpenOrdersAsString = string;
interface OpenOrderForPnl {
    nativeQuantityReleased: number;
    nativeQuantityPaid: number;
    side: 'sell' | 'buy';
    size: number;
    openOrders: OpenOrdersAsString;
}
export declare class MarketBalance {
    baseTokenSymbol: string;
    orders: number;
    unsettled: number;
    quoteTokenSymbol: string;
    quoteOrders: number;
    quoteUnsettled: number;
    constructor(baseTokenSymbol: string, orders: number, unsettled: number, quoteTokenSymbol: string, quoteOrders: number, quoteUnsettled: number);
}
export declare class MarginAccountBalance {
    tokenSymbol: string;
    deposited: number;
    borrowed: number;
    constructor(tokenSymbol: string, deposited: number, borrowed: number);
}
export declare class Balance {
    marginAccountPublicKey: string;
    marginAccountBalances: MarginAccountBalance[];
    marketBalances: MarketBalance[];
    constructor(marginAccountPublicKey: string, marginAccountBalances: MarginAccountBalance[], marketBalances: MarketBalance[]);
}
export declare class FetchMarketSymbol {
    symbol: string;
    constructor(symbol: string);
}
export declare class FetchMarket {
    symbols: FetchMarketSymbol[];
    constructor(symbols: FetchMarketSymbol[]);
}
export declare class Ticker {
    symbol: string;
    price: number;
    timeMs: number;
    constructor(symbol: string, price: number, timeMs: number);
}
export declare class Ohlcv {
    timeS: number;
    open: number;
    high: number;
    low: number;
    close: number;
    volume: number;
    constructor(timeS: number, open: number, high: number, low: number, close: number, volume: number);
}
declare type Resolution = '1' | '3' | '5' | '15' | '30' | '60' | '120' | '180' | '240' | '1D';
/**
 * a simpler more cex-style client with sensible (hopefully ;)) defaults
 */
export declare class SimpleClient {
    private client;
    private connection;
    private programId;
    private dexProgramId;
    private mangoGroup;
    private markets;
    private mangoGroupTokenMappings;
    private spotMarketMappings;
    private payer;
    private marginAccountPk;
    private constructor();
    static create(marginAccountPk: string): Promise<SimpleClient>;
    private getMarketForSymbol;
    private getMarginAccountForOwner;
    private getOpenOrdersAccountForSymbol;
    private cancelOrder;
    private cancelOrdersForMarginAccount;
    placeOrder(symbol: SpotMarketSymbol, type: 'market' | 'limit', side: 'buy' | 'sell', quantity: number, price?: number, orderType?: 'ioc' | 'postOnly' | 'limit'): Promise<string>;
    getOpenOrders(symbol: SpotMarketSymbol, clientId?: string): Promise<Order[]>;
    cancelOrders(symbol?: SpotMarketSymbol, clientId?: string): Promise<void>;
    getTradeHistory(symbol: SpotMarketSymbol): Promise<OpenOrderForPnl[]>;
    /**
     * returns balances, simple and mango specific details
     */
    getBalance(): Promise<Balance>;
    getPnl(): Promise<string>;
    /**
     * returns available markets
     */
    getMarkets(): Promise<FetchMarket>;
    /**
     * returns tickers i.e. symbol, closing price, time of closing price
     */
    getTickers(symbol?: SpotMarketSymbol): Promise<Ticker[]>;
    getOrderBook(symbol: SpotMarketSymbol): Promise<Order[]>;
    /**
     * returns ohlcv in ascending order for time
     */
    getOhlcv(spotMarketSymbol: SpotMarketSymbol, resolution: Resolution, fromEpochMs: number, toEpochMs: number): Promise<Ohlcv[]>;
}
export {};
//# sourceMappingURL=simpleclient.d.ts.map