"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const simpleclient_1 = require("../src/simpleclient");
const web3_js_1 = require("@solana/web3.js");
const test_utils_1 = require("./test_utils");
const serum_1 = require("@project-serum/serum");
const os_1 = __importDefault(require("os"));
const fs_1 = __importDefault(require("fs"));
const expect_1 = __importDefault(require("expect"));
const mango_client_1 = require("@blockworks-foundation/mango-client");
process.env.CLUSTER = 'devnet';
process.env.KEYPAIR = __dirname + '/' + 'id.json.bak';
const marketSymbol = 'BTC/USDC';
const lowPriceThatDoesntTrigger = 1;
const marginAccountPk = 'tBhXVv9JVJL8tApoBxEcKEgs7Ngd1FgdYGwBTarN4Ux';
const keyFilePath = process.env.KEYPAIR || os_1.default.homedir() + '/.config/solana/id.json';
describe('test simple client', async () => {
    (!fs_1.default.existsSync(keyFilePath) ? it : it.skip)('boostrap wallet and funds for trading', async () => {
        async function getSpotMarketDetails(mangoGroupSpotMarket) {
            const [spotMarketSymbol, spotMarketAddress] = mangoGroupSpotMarket;
            const [baseSymbol, quoteSymbol] = spotMarketSymbol.split('/');
            const spotMarket = await serum_1.Market.load(connection, new web3_js_1.PublicKey(spotMarketAddress), { skipPreflight: true, commitment: 'singleGossip' }, dexProgramId);
            return { spotMarket, baseSymbol, quoteSymbol };
        }
        // 1. mango specific setup
        const cluster = 'devnet';
        const clusterIds = mango_client_1.IDS[cluster];
        const dexProgramId = new web3_js_1.PublicKey(clusterIds.dex_program_id);
        const connection = new web3_js_1.Connection(mango_client_1.IDS.cluster_urls[cluster], 'singleGossip');
        const client = new mango_client_1.MangoClient();
        const mangoProgramId = new web3_js_1.PublicKey(clusterIds.mango_program_id);
        const mangoGroupName = 'BTC_ETH_SOL_SRM_USDC';
        const mangoGroupIds = clusterIds.mango_groups[mangoGroupName];
        const mangoGroupSpotMarkets = Object.entries(mangoGroupIds.spot_market_symbols);
        const mangoGroupPk = new web3_js_1.PublicKey(mangoGroupIds.mango_group_pk);
        const mangoGroup = await client.getMangoGroup(connection, mangoGroupPk);
        const mangoGroupTokenMappings = await test_utils_1.createMangoGroupSymbolMappings(connection, mangoGroupIds);
        // 2. create wallet
        let owner = await test_utils_1.createWalletAndRequestAirdrop(connection, 100);
        fs_1.default.writeFileSync(keyFilePath, `[${owner.secretKey.toString()}]`);
        owner = new web3_js_1.Account(JSON.parse(fs_1.default.readFileSync(keyFilePath, 'utf-8')));
        // 3. get margin accounts, if none exist then init one
        const prices = await mangoGroup.getPrices(connection);
        let marginAccounts = await client.getMarginAccountsForOwner(connection, mangoProgramId, mangoGroup, owner);
        marginAccounts.sort((a, b) => a.computeValue(mangoGroup, prices) > b.computeValue(mangoGroup, prices)
            ? -1
            : 1);
        if (marginAccounts.length === 0) {
            await client.initMarginAccount(connection, mangoProgramId, mangoGroup, owner);
            marginAccounts = await client.getMarginAccountsForOwner(connection, mangoProgramId, mangoGroup, owner);
        }
        // 4. deposit usdc
        await test_utils_1.createTokenAccountWithBalance(connection, owner, 'USDC', mangoGroupTokenMappings, clusterIds.faucets, 100000);
        await test_utils_1.performSingleDepositOrWithdrawal(connection, owner, client, mangoGroup, mangoProgramId, 'USDC', mangoGroupTokenMappings, marginAccounts[0], 'deposit', 100000);
        // 5. deposit base currency for each spot market
        for (const mangoGroupSpotMarket of mangoGroupSpotMarkets) {
            const { spotMarket, baseSymbol, quoteSymbol } = await getSpotMarketDetails(mangoGroupSpotMarket);
            const [orderSize, orderPrice, _] = await test_utils_1.getOrderSizeAndPrice(connection, spotMarket, mangoGroupTokenMappings, baseSymbol, quoteSymbol, 'buy');
            const neededQuoteAmount = orderPrice * orderSize;
            const neededQuoteAmountForAllTrades = neededQuoteAmount * 128;
            await test_utils_1.createTokenAccountWithBalance(connection, owner, baseSymbol, mangoGroupTokenMappings, clusterIds.faucets, neededQuoteAmountForAllTrades);
            await test_utils_1.performSingleDepositOrWithdrawal(connection, owner, client, mangoGroup, mangoProgramId, baseSymbol, mangoGroupTokenMappings, marginAccounts[0], 'deposit', neededQuoteAmountForAllTrades);
        }
    });
    async function cleanup(sc) {
        await sc.cancelOrders();
    }
    (process.env.CLUSTER === 'devnet' ? it : it.skip)('clean slate for devnet', async () => {
        const sc = await simpleclient_1.SimpleClient.create(marginAccountPk);
        await cleanup(sc);
        const orders = await sc.getOpenOrders(marketSymbol);
        expect_1.default(orders.length).toBe(0);
    });
    it('test place order', async () => {
        const sc = await simpleclient_1.SimpleClient.create(marginAccountPk);
        const ordersBeforePlacement = await sc.getOpenOrders(marketSymbol);
        expect_1.default(ordersBeforePlacement.length).toBe(0);
        await sc.placeOrder(marketSymbol, 'limit', 'buy', 0.0001, lowPriceThatDoesntTrigger);
        await sc.placeOrder(marketSymbol, 'limit', 'buy', 0.0002, lowPriceThatDoesntTrigger);
        const ordersAfterPlacement = await sc.getOpenOrders(marketSymbol);
        expect_1.default(ordersAfterPlacement.length).toBe(2);
        await cleanup(sc);
    });
    //
    it('test cancel a specific order', async () => {
        const sc = await simpleclient_1.SimpleClient.create(marginAccountPk);
        const txClientId = await sc.placeOrder(marketSymbol, 'limit', 'buy', 0.0001, lowPriceThatDoesntTrigger);
        const ordersBeforeCancellation = await sc.getOpenOrders(marketSymbol);
        await sc.cancelOrders(marketSymbol, txClientId);
        const ordersAfterCancellation = await sc.getOpenOrders(marketSymbol);
        expect_1.default(ordersAfterCancellation.length).toBe(ordersBeforeCancellation.length - 1);
    });
    it('test balances', async () => {
        const sc = await simpleclient_1.SimpleClient.create(marginAccountPk);
        const balance = await sc.getBalance();
        balance.marginAccountBalances.map((marginAccountBalance) => {
            expect_1.default(marginAccountBalance.deposited).toBeGreaterThan(0);
        });
    });
    (process.env.CLUSTER === 'mainnet-beta' ? it : it.skip)('test trade history', async () => {
        const sc = await simpleclient_1.SimpleClient.create(marginAccountPk);
        const tradeHistory = await sc.getTradeHistory(marketSymbol);
        expect_1.default(tradeHistory.length).toBeGreaterThan(0);
    });
    (process.env.CLUSTER === 'mainnet-beta' ? it : it.skip)('test tickers', async () => {
        const sc = await simpleclient_1.SimpleClient.create(marginAccountPk);
        const tickers = await sc.getTickers();
        expect_1.default(tickers.length).toBeGreaterThan(0);
    });
    (process.env.CLUSTER === 'mainnet-beta' ? it : it.skip)('test ohlcv', async () => {
        const sc = await simpleclient_1.SimpleClient.create(marginAccountPk);
        const to = Date.now();
        const yday = to - 24 * 60 * 60 * 1000;
        const ohlcvs = await sc.getOhlcv(marketSymbol, '1D', yday, to);
        expect_1.default(ohlcvs.length).toBeGreaterThan(0);
    });
    (process.env.CLUSTER === 'mainnet-beta' ? it : it.skip)('test pnl', async () => {
        const sc = await simpleclient_1.SimpleClient.create(marginAccountPk);
        const pnl = await sc.getPnl();
        expect_1.default(pnl).toBeGreaterThan(0);
    });
});
//# sourceMappingURL=simpleclient.test.js.map