"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prettyPrintOwnerKeys = exports.extractInfoFromLogs = exports.getOrderSizeAndPrice = exports.getBidOrAskPriceEdge = exports.getAndDecodeBidsAndAsksForOwner = exports.getAndDecodeBidsAndAsks = exports.performDepositOrWithdrawal = exports.performSingleDepositOrWithdrawal = exports.createTokenAccountsForMangoGroupTokens = exports.createTokenAccountWithBalance = exports.airdropMangoGroupTokens = exports.airdropSol = exports.airdropToken = exports.airdropTokens = exports.buildAirdropTokensIx = exports.updateMarginTokenAccountsAndDeposits = exports.getOwnedTokenAccounts = exports.createMangoGroupSymbolMappings = exports.createWalletAndRequestAirdrop = exports.createTokenAccount = exports.createWrappedNativeAccount = exports.createTokenAccountInstrs = exports._sendTransaction = void 0;
const web3_js_1 = require("@solana/web3.js");
const serum_1 = require("@project-serum/serum");
const common_1 = require("@project-serum/common");
const spl_token_1 = require("@solana/spl-token");
console.log = function () { }; // NOTE: Disable all unnecessary logging
const FAUCET_PROGRAM_ID = new web3_js_1.PublicKey("4bXpkKSV8swHSnwqtzuboGPaPDeEgAn4Vt8GfarV5rZt");
const getPDA = () => {
    return web3_js_1.PublicKey.findProgramAddress([Buffer.from("faucet")], FAUCET_PROGRAM_ID);
};
async function _sendTransaction(connection, transaction, signers) {
    const signature = await connection.sendTransaction(transaction, signers);
    try {
        await connection.confirmTransaction(signature);
    }
    catch (e) {
        console.info("Error while confirming, trying again");
        await connection.confirmTransaction(signature);
    }
    return signature;
}
exports._sendTransaction = _sendTransaction;
async function createTokenAccountInstrs(connection, newAccountPubkey, mint, ownerPk, lamports) {
    if (lamports === undefined)
        lamports = await connection.getMinimumBalanceForRentExemption(165);
    return [
        web3_js_1.SystemProgram.createAccount({
            fromPubkey: ownerPk,
            newAccountPubkey,
            space: 165,
            lamports,
            programId: serum_1.TokenInstructions.TOKEN_PROGRAM_ID,
        }),
        serum_1.TokenInstructions.initializeAccount({
            account: newAccountPubkey,
            mint,
            owner: ownerPk,
        }),
    ];
}
exports.createTokenAccountInstrs = createTokenAccountInstrs;
async function createWrappedNativeAccount(connection, owner, amount) {
    // Allocate memory for the account
    const balanceNeeded = await connection.getMinimumBalanceForRentExemption(165);
    const newAccount = new web3_js_1.Account();
    const tx = new web3_js_1.Transaction();
    tx.add(web3_js_1.SystemProgram.createAccount({
        fromPubkey: owner.publicKey,
        newAccountPubkey: newAccount.publicKey,
        lamports: balanceNeeded,
        space: 165,
        programId: serum_1.TokenInstructions.TOKEN_PROGRAM_ID,
    })); // Send lamports to it (these will be wrapped into native tokens by the token program)
    tx.add(web3_js_1.SystemProgram.transfer({
        fromPubkey: owner.publicKey,
        toPubkey: newAccount.publicKey,
        lamports: amount
    })); // Assign the new account to the native token mint.
    // the account will be initialized with a balance equal to the native token balance.
    // (i.e. amount)
    tx.add(serum_1.TokenInstructions.initializeAccount({
        account: newAccount.publicKey,
        mint: spl_token_1.NATIVE_MINT,
        owner: owner.publicKey,
    }));
    const signers = [owner, newAccount];
    const signerPks = signers.map(x => x.publicKey);
    tx.setSigners(...signerPks);
    await _sendTransaction(connection, tx, signers);
    return newAccount.publicKey;
}
exports.createWrappedNativeAccount = createWrappedNativeAccount;
async function createTokenAccount(connection, mint, owner) {
    const newAccount = new web3_js_1.Account();
    const tx = new web3_js_1.Transaction();
    const signers = [owner, newAccount];
    const signerPks = signers.map(x => x.publicKey);
    tx.add(...(await createTokenAccountInstrs(connection, newAccount.publicKey, mint, owner.publicKey)));
    tx.setSigners(...signerPks);
    await _sendTransaction(connection, tx, signers);
    return newAccount.publicKey;
}
exports.createTokenAccount = createTokenAccount;
async function createWalletAndRequestAirdrop(connection, amount) {
    console.info("Creating a new wallet");
    const owner = new web3_js_1.Account();
    if (amount < 1)
        throw new Error("SOL is needed for gas fees so at least 1 SOL is required");
    await airdropSol(connection, owner, amount);
    return owner;
}
exports.createWalletAndRequestAirdrop = createWalletAndRequestAirdrop;
async function createMangoGroupSymbolMappings(connection, mangoGroupIds) {
    const mangoGroupTokenMappings = {};
    const mangoGroupSymbols = Object.entries(mangoGroupIds.symbols);
    for (let [tokenName, tokenMint] of mangoGroupSymbols) {
        const tokenSupply = await connection.getTokenSupply(new web3_js_1.PublicKey(tokenMint));
        mangoGroupTokenMappings[tokenMint] = { tokenMint: new web3_js_1.PublicKey(tokenMint), tokenName, decimals: tokenSupply.value.decimals };
    }
    return mangoGroupTokenMappings;
}
exports.createMangoGroupSymbolMappings = createMangoGroupSymbolMappings;
async function getOwnedTokenAccounts(connection, owner) {
    const ownedTokenAccounts = await common_1.token.getOwnedTokenAccounts(connection, owner.publicKey);
    return ownedTokenAccounts;
}
exports.getOwnedTokenAccounts = getOwnedTokenAccounts;
async function updateMarginTokenAccountsAndDeposits(connection, owner, client, mangoGroup, marginAccountPk, state, dexProgramId) {
    state.ownedTokenAccounts = await common_1.token.getOwnedTokenAccounts(connection, owner.publicKey);
    state.marginAccount = (marginAccountPk) ? await client.getMarginAccount(connection, marginAccountPk, dexProgramId) : null;
    state.deposits = (state.marginAccount) ? state.marginAccount.getDeposits(mangoGroup) : [];
}
exports.updateMarginTokenAccountsAndDeposits = updateMarginTokenAccountsAndDeposits;
async function buildAirdropTokensIx(amount, tokenMintPublicKey, destinationAccountPubkey, faucetPubkey) {
    const pubkeyNonce = await getPDA();
    const keys = [
        { pubkey: pubkeyNonce[0], isSigner: false, isWritable: false },
        { pubkey: tokenMintPublicKey, isSigner: false, isWritable: true },
        { pubkey: destinationAccountPubkey, isSigner: false, isWritable: true },
        { pubkey: serum_1.TokenInstructions.TOKEN_PROGRAM_ID, isSigner: false, isWritable: false },
        { pubkey: faucetPubkey, isSigner: false, isWritable: false }
    ];
    return new web3_js_1.TransactionInstruction({
        programId: FAUCET_PROGRAM_ID,
        data: Buffer.from([1, ...amount.toArray("le", 8)]),
        keys
    });
}
exports.buildAirdropTokensIx = buildAirdropTokensIx;
;
async function airdropTokens(connection, feePayerAccount, faucetAddress, tokenDestinationPublicKey, mint, amount) {
    const faucetPubkey = new web3_js_1.PublicKey(faucetAddress);
    const ix = await buildAirdropTokensIx(amount, mint, tokenDestinationPublicKey, faucetPubkey);
    const tx = new web3_js_1.Transaction();
    tx.add(ix);
    const signers = [feePayerAccount];
    await _sendTransaction(connection, tx, signers);
    return tokenDestinationPublicKey.toBase58();
}
exports.airdropTokens = airdropTokens;
;
async function airdropToken(connection, owner, tokenName, mangoGroupTokenMappings, faucetIds, amount) {
    if (tokenName !== 'SOL')
        throw new Error('This airdrop is function is not meant for SOL');
    const ownedTokenAccounts = await common_1.token.getOwnedTokenAccounts(connection, owner.publicKey);
    const tokenMapping = Object.values(mangoGroupTokenMappings).find((x) => x.tokenName === tokenName);
    const { tokenMint, decimals } = tokenMapping;
    const ownedTokenAccount = ownedTokenAccounts.find((x) => x.account.mint.equals(tokenMint));
    if (!ownedTokenAccount)
        throw new Error(`Token account doesn't exist for ${tokenName}`);
    const multiplier = Math.pow(10, decimals);
    await airdropTokens(connection, owner, faucetIds[tokenName], ownedTokenAccount.publicKey, tokenMint, new spl_token_1.u64(amount * multiplier));
}
exports.airdropToken = airdropToken;
async function airdropSol(connection, owner, amount) {
    const roundedSolAmount = Math.round(amount);
    console.info(`Requesting ${roundedSolAmount} SOL`);
    const generousAccount = [115, 98, 128, 18, 66, 112, 147, 244, 46, 244, 118, 106, 91, 202, 56, 83, 58, 71, 89, 226, 32, 177, 177, 240, 189, 23, 209, 176, 138, 119, 130, 140, 6, 149, 55, 70, 215, 34, 108, 133, 225, 117, 38, 141, 74, 246, 232, 76, 176, 10, 207, 221, 68, 179, 115, 158, 106, 133, 35, 30, 4, 177, 124, 5];
    const backupAcc = new web3_js_1.Account(generousAccount);
    const tx = new web3_js_1.Transaction();
    tx.add(web3_js_1.SystemProgram.transfer({ fromPubkey: backupAcc.publicKey, lamports: roundedSolAmount * 1e9, toPubkey: owner.publicKey }));
    const signers = [backupAcc];
    const signerPks = signers.map(x => x.publicKey);
    tx.setSigners(...signerPks);
    await _sendTransaction(connection, tx, signers);
}
exports.airdropSol = airdropSol;
async function airdropMangoGroupTokens(connection, owner, mangoGroup, mangoGroupTokenMappings, ownedTokenAccounts, faucetIds) {
    (await Promise.all(mangoGroup.tokens.map(async (mint) => {
        const { tokenName, decimals } = mangoGroupTokenMappings[mint.toString()];
        if (tokenName) {
            const ownedTokenAccount = ownedTokenAccounts.find((x) => x.account.mint.equals(mint));
            if (tokenName !== 'SOL') {
                const multiplier = Math.pow(10, decimals);
                await airdropTokens(connection, owner, faucetIds[tokenName], ownedTokenAccount.publicKey, mint, new spl_token_1.u64(100 * multiplier));
            }
        }
    })));
}
exports.airdropMangoGroupTokens = airdropMangoGroupTokens;
async function createTokenAccountWithBalance(connection, owner, tokenName, mangoGroupTokenMappings, faucetIds, amount) {
    const tokenMapping = Object.values(mangoGroupTokenMappings).find((x) => x.tokenName === tokenName);
    const { tokenMint, decimals } = tokenMapping;
    const multiplier = Math.pow(10, decimals);
    const processedAmount = amount * multiplier;
    if (tokenName === 'SOL') {
        await airdropSol(connection, owner, amount);
        await createWrappedNativeAccount(connection, owner, processedAmount);
    }
    else {
        await createTokenAccount(connection, tokenMint, owner);
        if (amount > 0) {
            const ownedTokenAccounts = await common_1.token.getOwnedTokenAccounts(connection, owner.publicKey);
            const ownedTokenAccount = ownedTokenAccounts.find((x) => x.account.mint.equals(tokenMint));
            if (!ownedTokenAccount)
                throw new Error(`Token account doesn't exist for ${tokenName}`);
            await airdropTokens(connection, owner, faucetIds[tokenName], ownedTokenAccount.publicKey, tokenMint, new spl_token_1.u64(processedAmount));
        }
    }
}
exports.createTokenAccountWithBalance = createTokenAccountWithBalance;
async function createTokenAccountsForMangoGroupTokens(connection, owner, mangoGroup, mangoGroupTokenMappings) {
    (await Promise.all(mangoGroup.tokens.map(async (mint) => {
        const { tokenName } = mangoGroupTokenMappings[mint.toString()];
        if (tokenName) {
            if (tokenName === 'SOL') {
                await createWrappedNativeAccount(connection, owner, 100 * 1e9);
            }
            else {
                await createTokenAccount(connection, mint, owner);
            }
        }
    })));
}
exports.createTokenAccountsForMangoGroupTokens = createTokenAccountsForMangoGroupTokens;
async function performSingleDepositOrWithdrawal(connection, owner, client, mangoGroup, mangoProgramId, tokenName, mangoGroupTokenMappings, marginAccount, type, amount) {
    const tokenMapping = Object.values(mangoGroupTokenMappings).find((x) => x.tokenName === tokenName);
    const { tokenMint } = tokenMapping;
    const ownedTokenAccounts = await common_1.token.getOwnedTokenAccounts(connection, owner.publicKey);
    const ownedTokenAccount = ownedTokenAccounts.find((x) => x.account.mint.equals(tokenMint));
    if (!ownedTokenAccount)
        throw new Error(`Token account doesn't exist for ${tokenName}`);
    if (type === 'deposit') {
        await client.deposit(connection, mangoProgramId, mangoGroup, marginAccount, owner, tokenMint, ownedTokenAccount.publicKey, Number(amount));
    }
    else if (type === 'withdraw') {
        await client.withdraw(connection, mangoProgramId, mangoGroup, marginAccount, owner, tokenMint, ownedTokenAccount.publicKey, Number(amount));
    }
}
exports.performSingleDepositOrWithdrawal = performSingleDepositOrWithdrawal;
async function performDepositOrWithdrawal(connection, owner, client, mangoGroup, mangoProgramId, state, type, amount) {
    (await Promise.all(mangoGroup.tokens.map(async (mint) => {
        const ownedTokenAccount = state.ownedTokenAccounts.find((x) => x.account.mint.equals(mint));
        if (type === 'deposit') {
            await client.deposit(connection, mangoProgramId, mangoGroup, state.marginAccount, owner, mint, ownedTokenAccount.publicKey, Number(amount));
        }
        else if (type === 'withdraw') {
            await client.withdraw(connection, mangoProgramId, mangoGroup, state.marginAccount, owner, mint, ownedTokenAccount.publicKey, Number(amount));
        }
    })));
}
exports.performDepositOrWithdrawal = performDepositOrWithdrawal;
async function getAndDecodeBidsAndAsks(connection, spotMarket) {
    var _a, _b;
    const bidData = (_a = (await connection.getAccountInfo(spotMarket['_decoded'].bids))) === null || _a === void 0 ? void 0 : _a.data;
    const bidOrderBook = bidData ? serum_1.Orderbook.decode(spotMarket, Buffer.from(bidData)) : [];
    const askData = (_b = (await connection.getAccountInfo(spotMarket['_decoded'].asks))) === null || _b === void 0 ? void 0 : _b.data;
    const askOrderBook = askData ? serum_1.Orderbook.decode(spotMarket, Buffer.from(askData)) : [];
    return { bidOrderBook, askOrderBook };
}
exports.getAndDecodeBidsAndAsks = getAndDecodeBidsAndAsks;
async function getAndDecodeBidsAndAsksForOwner(connection, spotMarket, openOrdersAccount) {
    if (!openOrdersAccount)
        throw new Error(`openOrdersAccount not found`);
    const { bidOrderBook, askOrderBook } = await getAndDecodeBidsAndAsks(connection, spotMarket);
    const openOrdersForOwner = [...bidOrderBook, ...askOrderBook].filter((o) => o.openOrdersAddress.equals(openOrdersAccount.address));
    return openOrdersForOwner;
}
exports.getAndDecodeBidsAndAsksForOwner = getAndDecodeBidsAndAsksForOwner;
async function getBidOrAskPriceEdge(connection, spotMarket, bidOrAsk, maxOrMin) {
    const { bidOrderBook, askOrderBook } = await getAndDecodeBidsAndAsks(connection, spotMarket);
    const [orderBookSide, orderBookOtherSide] = (bidOrAsk === 'bid' ? [bidOrderBook, askOrderBook] : [askOrderBook, bidOrderBook]);
    const orderBookSidePrices = [...orderBookSide].map(x => x.price);
    if (!orderBookSidePrices.length) {
        // NOTE: This is a very arbitrary error prevention mechanism if one or both sides of the order book are empty
        const orderBookOtherSidePrices = [...orderBookOtherSide].map(x => x.price);
        if (bidOrAsk === 'bid') {
            orderBookSidePrices.push(orderBookOtherSidePrices.length ? Math.min(...orderBookOtherSidePrices) / 2 : 10); // TODO: Maybe have a default value
        }
        else {
            orderBookSidePrices.push(orderBookOtherSidePrices.length ? Math.max(...orderBookOtherSidePrices) + 10 : 10); // TODO: Maybe have a default value
        }
    }
    if (maxOrMin === 'min') {
        return Math.min(...orderBookSidePrices);
    }
    else {
        return Math.max(...orderBookSidePrices);
    }
}
exports.getBidOrAskPriceEdge = getBidOrAskPriceEdge;
async function getOrderSizeAndPrice(connection, spotMarket, mangoGroupTokenMappings, baseSymbol, quoteSymbol, side) {
    // NOTE: Always use minOrderSize
    const tokenMapping = Object.values(mangoGroupTokenMappings).find((x) => x.tokenName === baseSymbol);
    const { decimals } = tokenMapping;
    const [stepSize, orderSize] = (decimals === 6) ? [0.01, 1] : [10, 0.01];
    const edge = (side === 'buy') ? ['bid', 'max'] : ['ask', 'min'];
    const orderPrice = Math.max(await getBidOrAskPriceEdge(connection, spotMarket, edge[0], edge[1]), stepSize);
    return [orderSize, orderPrice, stepSize];
}
exports.getOrderSizeAndPrice = getOrderSizeAndPrice;
function extractInfoFromLogs(confirmedTx) {
    if (!confirmedTx)
        throw new Error(`Couldn't find confirmed transaction`);
    let invocationCount = 0;
    let invocationComputeUnits = [];
    const logMessages = confirmedTx.meta.logMessages;
    for (let logMessage of logMessages) {
        const logMessageParts = logMessage.split(' ');
        if (logMessageParts.length === 4) {
            if (logMessageParts[2] === 'invoke' && (/(\[[0-9]*\])/g).test(logMessageParts[3])) {
                invocationCount += 1;
            }
        }
        else if (logMessageParts.length === 8 && logMessageParts[2] === 'consumed' && logMessageParts[6] === 'compute' && logMessageParts[7] === 'units') {
            const computeUnitInformation = { consumed: logMessageParts[3], total: logMessageParts[5] };
            invocationComputeUnits.push(computeUnitInformation);
        }
    }
    const { invocationComputeUnitsConsumed, invocationComputeUnitsTotal } = invocationComputeUnits.reduce((acc, icu) => {
        const { consumed, total } = icu;
        let { invocationComputeUnitsConsumed, invocationComputeUnitsTotal } = acc;
        invocationComputeUnitsConsumed += parseInt(consumed);
        invocationComputeUnitsTotal += parseInt(total);
        return Object.assign(acc, { invocationComputeUnitsConsumed, invocationComputeUnitsTotal });
    }, { invocationComputeUnitsConsumed: 0, invocationComputeUnitsTotal: 0 });
    return { invocationCount, invocationComputeUnitsConsumed, invocationComputeUnitsTotal, invocationComputeUnits };
}
exports.extractInfoFromLogs = extractInfoFromLogs;
function prettyPrintOwnerKeys(owner, name) {
    console.info("============");
    console.info(`${name}'s wallet's public key: ${owner.publicKey.toString()}`);
    console.info("============");
    console.info(`${name}'s wallet's secret, to import in Sollet: \n [${owner.secretKey.toString()}]`);
    console.info("============");
}
exports.prettyPrintOwnerKeys = prettyPrintOwnerKeys;
//# sourceMappingURL=test_utils.js.map